@@ -190,7 +190,7 @@ class HttpParser(object):
         # end of body can be passed manually by putting a length of 0
 
         if length == 0:
-            self.on_message_complete = True
+            self.__on_message_complete = True
             return length
 
         # start to parse
@@ -340,11 +340,18 @@ class HttpParser(object):
             name = name.rstrip(" \t").upper()
             if HEADER_RE.search(name):
                 raise InvalidHeader("invalid header name %s" % name)
+
+            if value.endswith("\r\n"):
+                value = value[:-2]
+
             name, value = name.strip(), [value.lstrip()]
 
             # Consume value continuation lines
             while len(lines) and lines[0].startswith((" ", "\t")):
-                value.append(lines.pop(0))
+                curr = lines.pop(0)
+                if curr.endswith("\r\n"):
+                    curr = curr[:-2]
+                value.append(curr)
             value = ''.join(value).rstrip()
 
             # multiple headers
@@ -374,10 +381,11 @@ class HttpParser(object):
 
         # detect encoding and set decompress object
         encoding = self._headers.get('content-encoding')
-        if encoding == "gzip":
-            self.__decompress_obj = zlib.decompressobj(16+zlib.MAX_WBITS)
-        elif encoding == "deflate":
-            self.__decompress_obj = zlib.decompressobj()
+        if self.decompress:
+            if encoding == "gzip":
+                self.__decompress_obj = zlib.decompressobj(16+zlib.MAX_WBITS)
+            elif encoding == "deflate":
+                self.__decompress_obj = zlib.decompressobj()
 
         rest = data[idx+4:]
         self._buf = [rest]
